/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: AnnotationVisitorDispatcher.java 3600 2008-07-04 15:25:28Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.api.configurator.metadata.specific;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.scan.api.IAnnotationVisitor;

/**
 * AnnotationVisitorDispatcher.
 * Dispatch methods call in a list of visitor.
 * @author Gael Lalire
 */
public class AnnotationVisitorDispatcher implements IAnnotationVisitor {

    /**
     * List of visitor.
     */
    private List<IAnnotationVisitor> annotationVisitorList;

    /**
     * @param annotationVisitorList a visitor list
     */
    public AnnotationVisitorDispatcher(final List<IAnnotationVisitor> annotationVisitorList) {
        this.annotationVisitorList = annotationVisitorList;
    }

    /**
     * Dispatch visit method.
     * @param name name
     * @param value value
     */
    public void visit(final String name, final Object value) {
        for (IAnnotationVisitor annotationVisitor : annotationVisitorList) {
            annotationVisitor.visit(name, value);
        }
    }

    /**
     * Dispatch visitAnnotation method.
     * @param name name
     * @param desc desc
     * @return an annotation visitor
     */
    public IAnnotationVisitor visitAnnotation(final String name, final String desc) {
        List<IAnnotationVisitor> newAnnotationVisitorList = new ArrayList<IAnnotationVisitor>();
        for (IAnnotationVisitor annotationVisitor : annotationVisitorList) {
            newAnnotationVisitorList.add(annotationVisitor.visitAnnotation(name, desc));
        }
        return new AnnotationVisitorDispatcher(newAnnotationVisitorList);
    }

    /**
     * Dispatch visitArray method.
     * @param name name
     * @return an annotation visitor
     */
    public IAnnotationVisitor visitArray(final String name) {
        List<IAnnotationVisitor> newAnnotationVisitorList = new ArrayList<IAnnotationVisitor>();
        for (IAnnotationVisitor annotationVisitor : annotationVisitorList) {
            newAnnotationVisitorList.add(annotationVisitor.visitArray(name));
        }
        return new AnnotationVisitorDispatcher(newAnnotationVisitorList);
    }

    /**
     * Dispatch visitEnd method.
     */
    public void visitEnd() {
        for (IAnnotationVisitor annotationVisitor : annotationVisitorList) {
            annotationVisitor.visitEnd();
        }
    }

    /**
     * Dispatch visitEnum method.
     * @param name name
     * @param desc desc
     * @param value value
     */
    public void visitEnum(final String name, final String desc, final String value) {
        for (IAnnotationVisitor annotationVisitor : annotationVisitorList) {
            annotationVisitor.visitEnum(name, desc, value);
        }
    }

}
