/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: FieldMetadata.java 3253 2008-05-16 12:47:44Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.api.metadata;

import org.ow2.util.scan.api.metadata.structures.JField;

/**
 * This class represents the annotation metadata of a field.
 * @author Florent Benoit
 * @param <C> type of class metadata
 * @param <M> type of method metadata
 * @param <F> type of field metadata
 */
public abstract class FieldMetadata<
    C extends IClassMetadata<C, M, F>,
    M extends IMethodMetadata<C, M, F>,
    F extends IFieldMetadata<C, M, F>>
        extends CommonMetadata implements IFieldMetadata<C, M, F> {

    /**
     * Logger.
     */
    //private static Log logger = LogFactory.getLog(FieldAnnotationMetadata.class);

    /**
     * Method on which we got metadata.
     */
    private JField jField = null;

    /**
     * Parent metadata.
     */
    private C classAnnotationMetadata = null;

    /**
     * This field is a field from a super class ?<br>
     */
    private boolean inherited = false;


    /**
     * Constructor.
     * @param jField the field on which we will set/add metadata
     * @param classAnnotationMetadata the parent metadata.
     */
    public FieldMetadata(final JField jField, final C classAnnotationMetadata) {
        this.jField = jField;
        this.classAnnotationMetadata = classAnnotationMetadata;
    }

    /**
     * @return JMethod object
     */
    public final JField getJField() {
        return this.jField;
    }

    /**
     * @return true if this method is inherited from a super class
     */
    public final boolean isInherited() {
        return inherited;
    }

    /**
     * Sets the inheritance of this method.
     * @param inherited true if method is from a super class
     */
    public final void setInherited(final boolean inherited) {
        this.inherited = inherited;
    }

    /**
     * @return parent metadata (class)
     */
    public final C getClassMetadata() {
        return classAnnotationMetadata;
    }

    /**
     * Clone this object and attach it to class metadata.
     * @param classMetadata the class metadata
     * @return the clone
     * @throws CloneNotSupportedException never
     */
    @SuppressWarnings("unchecked")
    public F attachClone(final C classMetadata) throws CloneNotSupportedException {
        FieldMetadata<C, M, F> fieldMetadata = (FieldMetadata<C, M, F>) clone();
        fieldMetadata.classAnnotationMetadata = classMetadata;
        return (F) fieldMetadata;
    }


}
