package org.ow2.util.scan.api.metadata.structures;

/**
 * Annotation on class, method or field.
 * @author Gael Lalire
 */
public final class JAnnotation {

    /**
     * annotation name.
     */
    private String className;

    /**
     * visibility of annotation.
     */
    private boolean visible;

    /**
     * Constructor.
     * @param className the class name of annotation
     * @param visible visible
     */
    public JAnnotation(final String className, final boolean visible) {
        if (className == null) {
            throw new IllegalArgumentException("Class name argument of JAnnotation constructor can not be null");
        }
        this.className = className;
        this.visible = visible;
    }

    /**
     * @return visibility of annotation
     */
    public boolean isVisible() {
        return visible;
    }

    /**
     * @return name of annotation class
     */
    public String getClassName() {
        return className;
    }

    /**
     * @return name hashcode
     */
    @Override
    public int hashCode() {
        return className.hashCode();
    }

    /**
     * @param obj the object to compare
     * @return true if same annotation.
     */
    @Override
    public boolean equals(final Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof JAnnotation)) {
            return false;
        } else {
            JAnnotation jAnnotation = (JAnnotation) obj;
            if (className.equals(jAnnotation.className) && visible == jAnnotation.visible) {
                return true;
            }
            return false;
        }
    }

}
